@@ -714,11 +714,9 @@ static void
 basic_engine_shape (PangoEngineShape 	*engine,
 		    PangoFont        	*font,
 		    const char       	*text,
-		    unsigned int     	 length,
+		    int              	 length,
 		    const PangoAnalysis *analysis,
-		    PangoGlyphString    *glyphs,
-		    const char          *paragraph_text G_GNUC_UNUSED,
-		    unsigned int         paragraph_length G_GNUC_UNUSED)
+		    PangoGlyphString    *glyphs)
 {
   int n_chars;
   int i;
